

local utilities = require("utilities")
local interface = require("interface")

-- Initialize global variables
local function initialize_globals()
    global.heat_engines = global.heat_engines or {}
    global.entities = global.entities or {}
    global.ticker = global.ticker or 0
end

script.on_init(initialize_globals)
script.on_configuration_changed(initialize_globals)

script.on_event(defines.events.on_tick, function(event)
    global.ticker = global.ticker + 1
end)

script.on_event(defines.events.on_built_entity, function(event)
    local created_entity = event.created_entity
    if created_entity then
        if created_entity.name == "heat-engine" then
            local position = created_entity.position
            local surface = created_entity.surface
            local energy_interface = surface.create_entity({
                name = "hidden-energy-interface",
                type = "electric-energy-interface",
                position = {position.x + 1, position.y},
                force = created_entity.force,
                create_build_effect_smoke = false,
                render_player_index = event.player_index,
                visible = false
            })

            if energy_interface then
                table.insert(global.heat_engines, {engine = created_entity, interface = energy_interface})
            else
                game.print("Failed to create hidden energy interface.")
            end
        end
    end
end)

script.on_nth_tick(60, function()
    for _, engine_data in pairs(global.heat_engines) do
        local heat_engine = engine_data.engine
        if heat_engine and heat_engine.valid then
            if heat_engine.type == "heat-pipe" then
                local temperature = utilities.get_heat_engine_temperature(heat_engine)
                if temperature then
                    engine_data.temperature = temperature  -- Store the temperature
                else
                    game.print("Could not get temperature from the heat engine.")
                end
            end
        end
    end
end)

script.on_nth_tick(90, function()
    for _, engine_data in pairs(global.heat_engines) do
        local temperature = engine_data.temperature
        local energy_interface = engine_data.interface
        if temperature and energy_interface and energy_interface.valid then
            local energyOutput = interface.calculate_energy_output(temperature)
            utilities.update_flow_limits(energy_interface, energyOutput)
            utilities.update_energy_production(energy_interface, energyOutput)
        else
            game.print("Energy interface is invalid or temperature is missing.")
        end
    end
end)
