local utilities = require("utilities")
local ENTITYPATH = "__heat-engine__/graphics/entity/"

local function single_sprite(filename)
	return {
		filename = ENTITYPATH .. filename,
		width = 123,
		height = 84,
		hr_version = {
			filename = ENTITYPATH .. filename,
			width = 123,
			height = 84,
		}
	}
end

local pipe_sprites = {
	straight_vertical = single_sprite("heat-engine.png"),
	straight_horizontal = single_sprite("heat-engine.png"),
	corner_right_down = single_sprite("heat-engine.png"),
	corner_left_down = single_sprite("heat-engine.png"),
	corner_right_up = single_sprite("heat-engine.png"),
	corner_left_up = single_sprite("heat-engine.png"),
	t_up = single_sprite("heat-engine.png"),
	t_right = single_sprite("heat-engine.png"),
	t_down = single_sprite("heat-engine.png"),
	t_left = single_sprite("heat-engine.png"),
	ending_up = single_sprite("heat-engine.png"),
	ending_right = single_sprite("heat-engine.png"),
	ending_down = single_sprite("heat-engine.png"),
	ending_left = single_sprite("heat-engine.png"),
	cross = single_sprite("heat-engine.png"),
	single = single_sprite("heat-engine.png")
}

local heat_sprites = {
	straight_vertical = single_sprite("heat-engineglow.png"),
	straight_horizontal = single_sprite("heat-engineglow.png"),
	corner_right_down = single_sprite("heat-engineglow.png"),
	corner_left_down = single_sprite("heat-engineglow.png"),
	corner_right_up = single_sprite("heat-engineglow.png"),
	corner_left_up = single_sprite("heat-engineglow.png"),
	t_up = single_sprite("heat-engineglow.png"),
	t_right = single_sprite("heat-engineglow.png"),
	t_down = single_sprite("heat-engineglow.png"),
	t_left = single_sprite("heat-engineglow.png"),
	ending_up = single_sprite("heat-engineglow.png"),
	ending_right = single_sprite("heat-engineglow.png"),
	ending_down = single_sprite("heat-engineglow.png"),
	ending_left = single_sprite("heat-engineglow.png"),
	cross = single_sprite("heat-engineglow.png"),
	single = single_sprite("heat-engineglow.png")
}

local function createConnections(connection_type)
    return {
        {
            type = connection_type,
            position = {0, -1},
            direction = defines.direction.north
        },
        {
            type = connection_type,
            position = {0, 1},
            direction = defines.direction.south
        },
        {
            type = connection_type,
            position = {-1, 0},
            direction = defines.direction.west
        },
        {
            type = connection_type,
            position = {1, 0},
            direction = defines.direction.east
        }
    }
end

data:extend({
    {
        type = "electric-energy-interface",
        name = "hidden-energy-interface",
        icon = "__base__/graphics/icons/accumulator.png",
        icon_size = 64,
        flags = {"placeable-neutral", "player-creation", "hidden"},
        max_health = 100,
        collision_box = {{-0.5, -0.5}, {0.5, 0.5}},
        collision_mask = {},
        selection_box = {{-0.5, -0.5}, {0.5, 0.5}},
        energy_source = {
            type = "electric",
            usage_priority = "primary-output",
            buffer_capacity = "5MJ",
            input_flow_limit = "0W",
            output_flow_limit = "0W",
            drain = "0W"
        },
        energy_production = "1W",
        energy_usage = "0kW",
        picture = {
            filename = "__heat-engine__/graphics/entity/heat-engineglow.png",
            priority = "extra-high",
            width = 1,
            height = 1
        },
        connection_points = {
            {
                shadow = {
                    copper = {0, 0},
                    green = {0, 0},
                    red = {0, 0}
                },
                wire = {
                    copper = {0, 0},
                    green = {0, 0},
                    red = {0, 0}
                }
            }
        },
        copper_wire_picture = {
            filename = "__base__/graphics/entity/small-electric-pole/copper-wire.png",
            priority = "extra-high",
            width = 224,
            height = 46,
            direction_count = 1,
            shift = {0, -0.71875}
        },
        circuit_wire_connection_point = {
            shadow = {
                red = {0, 0},
                green = {0, 0}
            },
            wire = {
                red = {0, 0},
                green = {0, 0}
            },
        },
        circuit_wire_max_distance = 9,
        default_output_signal = nil,
        vehicle_impact_sound = {filename = "__base__/sound/car-metal-impact.ogg", volume = 0.65}
    },
    {
        type = "heat-pipe",
        name = "heat-engine",
        icon = "__heat-engine__/graphics/icons/heat-engine.png",
        icon_size = 64,
        flags = {"placeable-neutral", "player-creation"},
        minable = {mining_time = 1, result = "heat-engine"},
        max_health = 300,
        corpse = "medium-remnants",
        dying_explosion = "medium-explosion",
        collision_box = {{-1.2, -1.2}, {1.2, 1.2}},
        selection_box = {{-1.5, -1.5}, {1.5, 1.5}},
        picture = {
            layers = {
                {
                    filename = "__heat-engine__/graphics/entity/heat-engineH.png",
                    line_length = 4,
                    width = 120,
                    height = 84,
                    frame_count = 24,
                    animation_speed = 0.75,
                    shift = {0, 0},
                    hr_version = {
                        filename = "__heat-engine__/graphics/entity/heat-engineHR.png",
                        line_length = 4,
                        width = 120,
                        height = 84,
                        frame_count = 24,
                        animation_speed = 0.75,
                        shift = {0, 0},
                        scale = 0.5
                    }
                }
            }
        },
        connection_sprites = pipe_sprites,
        heat_glow_sprites = heat_sprites,
        minimum_glow_temperature = 200,
        heat_buffer = {
            max_temperature = 5000,
            specific_heat = "1GJ",
            heat_consumption_per_tick = "1KJ",
            max_transfer = "1GW",
            buffer_capacity = "1GJ",
            min_temperature = 15,},
        connections = createConnections("heat"),

        heat_glow_light = {
            minimum_darkness = 0.3,
            color = {r = 1, g = 155/255, b = 0.05, a = 0},
            shift = {-0.6, 3.5},
            size = 2.5,
            intensity = 0.3,
            add_perspective = true
       }
    },
    {
        type = "item",
        name = "heat-engine",
        icon = "__heat-engine__/graphics/icons/heat-engine.png",
        subgroup = "energy",
        order = "b[steam-power]-d[heat-engine]",
        place_result = "heat-engine",
        icon_size = 64,
        stack_size = 10
    },
    {
        type = "item",
        name = "hidden-energy-interface",
        icon =  "__heat-engine__/graphics/icons/steam-tech.png",
        subgroup = "energy",
        order = "b[steam-power]-d[heat-engine]",
        place_result = "hidden-energy-interface",
        icon_size = 64,
        stack_size = 10,
        hidden = true
    },
    {
        type = "recipe",
        name = "heat-engine",
        category = "crafting",
        normal = {
            ingredients = {
                {"steam-engine", 1},
                {"pipe", 5},
                {"iron-plate", 5}
            },
            result = "heat-engine"
        },
        expensive = {
            ingredients = {
                {"steam-engine", 2},
                {"pipe", 10},
                {"iron-plate", 10}
            },
            result = "heat-engine"
        }
    },
    {
        type = "technology",
        name = "heat-engine",
        icon = "__heat-engine__/graphics/icons/heat-engine.png",
        icon_size = 64,
        category = "basic-crafting",
        prerequisites = {"advanced-material-processing", "engine"},
        unit = {
            count = 100,
            ingredients = {
                {"automation-science-pack", 1},
                {"logistic-science-pack", 1}
            },
            time = 30
        },
        effects = {
            {
                type = "unlock-recipe",
                recipe = "heat-engine"
            }
        }
    }
})
